#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import pandas as pd
import shutil

from tests.test_streams.base import get_streams_client, set_store
from tests.utils import BaseTestCase

from polyaxon import settings
from polyaxon.polyboard.artifacts import V1ArtifactKind
from polyaxon.polyboard.events import V1Event, V1Events
from polyaxon.polyboard.events.schemas import LoggedEventListSpec
from polyaxon.streams.app.main import STREAMS_URL
from polyaxon.utils.path_utils import create_path


class TestEventsEndpoints(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.store_root = set_store()
        self.run_path = os.path.join(self.store_root, "uuid")
        self.run_events = os.path.join(self.run_path, "events")
        # Create run artifacts path
        create_path(self.run_path)
        # Create run artifacts events path
        create_path(self.run_events)
        # Create run events
        self.create_tmp_events()

        self.client = get_streams_client()
        self.base_url = STREAMS_URL + "/namespace/owner/project/runs/uuid/events"

    def create_tmp_events(self):
        text1 = LoggedEventListSpec(
            name="text1",
            kind=V1ArtifactKind.TEXT,
            events=[
                V1Event.make(step=1, text="foo1"),
                V1Event.make(step=2, text="boo2"),
            ],
        )
        self.create_kind_events(name="text1", kind=V1ArtifactKind.TEXT, events=text1)
        text2 = LoggedEventListSpec(
            name="text2",
            kind=V1ArtifactKind.TEXT,
            events=[
                V1Event.make(step=1, text="foo2"),
                V1Event.make(step=2, text="boo2"),
            ],
        )
        self.create_kind_events(name="text2", kind=V1ArtifactKind.TEXT, events=text2)
        html1 = LoggedEventListSpec(
            name="html1",
            kind=V1ArtifactKind.HTML,
            events=[
                V1Event.make(step=1, html="foo1"),
                V1Event.make(step=2, html="boo2"),
            ],
        )
        self.create_kind_events(name="html1", kind=V1ArtifactKind.HTML, events=html1)
        html2 = LoggedEventListSpec(
            name="htm2",
            kind=V1ArtifactKind.HTML,
            events=[
                V1Event.make(step=1, html="foo2"),
                V1Event.make(step=2, html="boo2"),
            ],
        )
        self.create_kind_events(name="html2", kind=V1ArtifactKind.HTML, events=html2)

    def create_kind_events(self, name, kind, events):
        event_kind_path = "{}/{}".format(self.run_events, kind)
        create_path(event_kind_path)
        with open("{}/{}.plx".format(event_kind_path, name), "+w") as f:
            f.write(events.get_csv_header())
            f.write(events.get_csv_events())

    def test_download_events_cached(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True

        shutil.rmtree(self.run_events)
        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True

        shutil.rmtree(
            os.path.join(settings.CLIENT_CONFIG.archive_root, "uuid", "events")
        )
        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False

    def test_download_events_cached_force(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True

        shutil.rmtree(self.run_events)
        response = self.client.get(
            self.base_url + "/text?names=text1,text2&orient=csv&force=true"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False

    def test_download_text_events_as_csv(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/text?names=text1&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is False
        events1 = V1Events.read(name="text1", kind=V1ArtifactKind.TEXT, data=filepath1)
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        assert results[0].name == events1.name
        assert results[0].kind == events1.kind
        assert pd.DataFrame.equals(results[0].df, events1.df)

        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        events2 = V1Events.read(name="text2", kind=V1ArtifactKind.TEXT, data=filepath2)
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url + "/text?names=text1,text2,text3&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

    def test_download_html_events_as_csv(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "html", "html1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "html", "html2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/html?names=html1&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is False
        events1 = V1Events.read(name="html1", kind=V1ArtifactKind.HTML, data=filepath1)
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        assert results[0].name == events1.name
        assert results[0].kind == events1.kind
        assert pd.DataFrame.equals(results[0].df, events1.df)

        response = self.client.get(self.base_url + "/html?names=html1,html2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        events2 = V1Events.read(name="html2", kind=V1ArtifactKind.HTML, data=filepath2)
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url + "/html?names=text1,html1,html2&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

    def test_download_text_events_as_dict(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/text?names=text1&orient=dict")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is False
        events1 = V1Events.read(
            name="text1", kind=V1ArtifactKind.TEXT, data=filepath1, parse_dates=False
        )
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        assert results[0].name == events1.name
        assert results[0].kind == events1.kind
        assert pd.DataFrame.equals(results[0].df, events1.df)

        response = self.client.get(
            self.base_url + "/text?names=text1,text2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        events2 = V1Events.read(
            name="text2", kind=V1ArtifactKind.TEXT, data=filepath2, parse_dates=False
        )
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url + "/text?names=text1,text2,text3&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

    def test_download_html_events_as_dict(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "html", "html1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "html", "html2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/html?names=html1&orient=dict")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is False
        events1 = V1Events.read(
            name="html1", kind=V1ArtifactKind.HTML, data=filepath1, parse_dates=False
        )
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        assert results[0].name == events1.name
        assert results[0].kind == events1.kind
        assert pd.DataFrame.equals(results[0].df, events1.df)

        response = self.client.get(
            self.base_url + "/html?names=html1,html2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        events2 = V1Events.read(
            name="html2", kind=V1ArtifactKind.HTML, data=filepath2, parse_dates=False
        )
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url + "/html?names=text1,html1,html2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)


class TestMultiRunsEventsEndpoints(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.store_root = set_store()
        self.run1_path = os.path.join(self.store_root, "uuid1")
        self.run1_events = os.path.join(self.run1_path, "events")
        self.run2_path = os.path.join(self.store_root, "uuid2")
        self.run2_events = os.path.join(self.run2_path, "events")
        # Create run artifacts path
        create_path(self.run1_path)
        create_path(self.run2_path)
        # Create run artifacts events path
        create_path(self.run1_events)
        create_path(self.run2_events)
        # Create run events
        self.create_tmp_events(run_events=self.run1_events)
        self.create_tmp_events(run_events=self.run2_events)

        self.client = get_streams_client()
        self.base_url = STREAMS_URL + "/namespace/owner/project/runs/multi/events"

    def create_tmp_events(self, run_events: str):
        metric1 = LoggedEventListSpec(
            name="metric1",
            kind=V1ArtifactKind.METRIC,
            events=[
                V1Event.make(step=1, metric=1.1),
                V1Event.make(step=2, metric=1.2),
            ],
        )
        self.create_kind_events(
            run_events=run_events,
            name="metric1",
            kind=V1ArtifactKind.METRIC,
            events=metric1,
        )
        metric2 = LoggedEventListSpec(
            name="metric2",
            kind=V1ArtifactKind.METRIC,
            events=[
                V1Event.make(step=1, metric=2.1),
                V1Event.make(step=2, metric=2.2),
            ],
        )
        self.create_kind_events(
            run_events=run_events,
            name="metric2",
            kind=V1ArtifactKind.METRIC,
            events=metric2,
        )

    def create_kind_events(self, run_events, name, kind, events):
        event_kind_path = "{}/{}".format(run_events, kind)
        create_path(event_kind_path)
        with open("{}/{}.plx".format(event_kind_path, name), "+w") as f:
            f.write(events.get_csv_header())
            f.write(events.get_csv_events())

    def test_download_multi_metric_events_cached(self):
        filepath11 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath12 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric2.plx",
        )
        filepath21 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath22 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric2.plx",
        )

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True

        shutil.rmtree(self.run1_path)
        shutil.rmtree(self.run2_path)

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True

        shutil.rmtree(os.path.join(settings.CLIENT_CONFIG.archive_root, "uuid1"))
        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is False
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True

    def test_download_multi_metric_events_cached_force(self):
        filepath11 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath12 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric2.plx",
        )
        filepath21 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath22 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric2.plx",
        )

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True

        shutil.rmtree(self.run1_path)
        shutil.rmtree(self.run2_path)

        response = self.client.get(
            self.base_url
            + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict&force=true"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is False
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False

    def test_download_multi_metric_events_as_dict(self):
        filepath11 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath12 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric2.plx",
        )
        filepath21 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath22 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric2.plx",
        )
        assert os.path.exists(filepath11) is False
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False
        response = self.client.get(
            self.base_url + "/metric?names=metric1&runs=uuid1&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False
        events11 = V1Events.read(
            name="metric1",
            kind=V1ArtifactKind.METRIC,
            data=filepath11,
            parse_dates=False,
        )
        results = response.json()["data"]
        assert list(results.keys()) == ["uuid1"]
        assert len(results["uuid1"]) == 1
        results = [V1Events.read(**i) for i in response.json()["data"]["uuid1"]]
        assert results[0].name == events11.name
        assert results[0].kind == events11.kind
        assert pd.DataFrame.equals(results[0].df, events11.df)

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True
        events12 = V1Events.read(
            name="metric2",
            kind=V1ArtifactKind.METRIC,
            data=filepath12,
            parse_dates=False,
        )
        events21 = V1Events.read(
            name="metric1",
            kind=V1ArtifactKind.METRIC,
            data=filepath21,
            parse_dates=False,
        )
        events22 = V1Events.read(
            name="metric2",
            kind=V1ArtifactKind.METRIC,
            data=filepath22,
            parse_dates=False,
        )
        expected = {
            "uuid1": {events11.name: events11, events12.name: events12},
            "uuid2": {events21.name: events21, events22.name: events22},
        }
        response = response.json()["data"]
        results = {
            run_uuid: [V1Events.read(**i) for i in response[run_uuid]]
            for run_uuid in response
        }
        for run_uuid in results:
            for res in results[run_uuid]:
                exp = expected[run_uuid][res.name]
                assert res.name == exp.name
                assert res.kind == exp.kind
                assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url
            + "/metric?names=metric12,text2,text3&runs=uuid1,uuid2,uuid3&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True
        expected = {
            "uuid1": {events11.name: events11, events12.name: events12},
            "uuid2": {events21.name: events21, events22.name: events22},
        }
        response = response.json()["data"]
        results = {
            run_uuid: [V1Events.read(**i) for i in response[run_uuid]]
            for run_uuid in response
        }
        for run_uuid in results:
            for res in results[run_uuid]:
                exp = expected[run_uuid][res.name]
                assert res.name == exp.name
                assert res.kind == exp.kind
                assert pd.DataFrame.equals(res.df, exp.df)

    def test_download_multi_metric_events_as_csv(self):
        filepath11 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath12 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric2.plx",
        )
        filepath21 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath22 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric2.plx",
        )
        assert os.path.exists(filepath11) is False
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False
        response = self.client.get(
            self.base_url + "/metric?names=metric1&runs=uuid1&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False
        events11 = V1Events.read(
            name="metric1", kind=V1ArtifactKind.METRIC, data=filepath11
        )
        results = response.json()["data"]
        assert list(results.keys()) == ["uuid1"]
        assert len(results["uuid1"]) == 1
        results = [V1Events.read(**i) for i in response.json()["data"]["uuid1"]]
        assert results[0].name == events11.name
        assert results[0].kind == events11.kind
        assert pd.DataFrame.equals(results[0].df, events11.df)

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True
        events12 = V1Events.read(
            name="metric2", kind=V1ArtifactKind.METRIC, data=filepath12
        )
        events21 = V1Events.read(
            name="metric1", kind=V1ArtifactKind.METRIC, data=filepath21
        )
        events22 = V1Events.read(
            name="metric2", kind=V1ArtifactKind.METRIC, data=filepath22
        )
        expected = {
            "uuid1": {events11.name: events11, events12.name: events12},
            "uuid2": {events21.name: events21, events22.name: events22},
        }
        response = response.json()["data"]
        results = {
            run_uuid: [V1Events.read(**i) for i in response[run_uuid]]
            for run_uuid in response
        }
        for run_uuid in results:
            for res in results[run_uuid]:
                exp = expected[run_uuid][res.name]
                assert res.name == exp.name
                assert res.kind == exp.kind
                assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url
            + "/metric?names=metric12,text2,text3&runs=uuid1,uuid2,uuid3&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True
        expected = {
            "uuid1": {events11.name: events11, events12.name: events12},
            "uuid2": {events21.name: events21, events22.name: events22},
        }
        response = response.json()["data"]
        results = {
            run_uuid: [V1Events.read(**i) for i in response[run_uuid]]
            for run_uuid in response
        }
        for run_uuid in results:
            for res in results[run_uuid]:
                exp = expected[run_uuid][res.name]
                assert res.name == exp.name
                assert res.kind == exp.kind
                assert pd.DataFrame.equals(res.df, exp.df)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class PolyaxonServiceHeaders:
    CLI_VERSION = "X_POLYAXON_CLI_VERSION"
    CLIENT_VERSION = "X_POLYAXON_CLIENT_VERSION"
    INTERNAL = "X_POLYAXON_INTERNAL"
    SERVICE = "X_POLYAXON_SERVICE"

    @staticmethod
    def get_header(header):
        return header.replace("_", "-")


class PolyaxonServices:  # noqa
    PLATFORM = "platform"
    AUTH = "auth"
    UI = "ui"
    CLI = "cli"
    INITIALIZER = "initializer"
    SIDECAR = "sidecar"
    RUNNER = "runner"
    AGENT = "agent"
    OPERATOR = "operator"

    VALUES = {PLATFORM, CLI, UI, AUTH, INITIALIZER, SIDECAR, RUNNER, AGENT, OPERATOR}
    AGENT_VALUES = [PLATFORM, CLI, UI, OPERATOR, AGENT, INITIALIZER, SIDECAR]



#!/usr/bin/env python
import os
import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


def read_readme():
    if not os.path.exists("../README.md"):
        return ""
    with open("../README.md") as f:
        return f.read()


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        errcode = pytest.main(self.test_args)
        sys.exit(errcode)


setup(
    name="polyaxon",
    version="1.1.6-post1",
    description="Command Line Interface (CLI) and client to interact with Polyaxon API.",
    long_description=read_readme(),
    long_description_content_type="text/markdown",
    maintainer="Polyaxon, Inc.",
    maintainer_email="contact@polyaxon.com",
    author="Polyaxon, Inc.",
    author_email="contact@polyaxon.com",
    url="https://github.com/polyaxon/polyaxon",
    license="Apache 2.0",
    platforms="any",
    packages=find_packages(),
    keywords=[
        "polyaxon",
        "aws",
        "s3",
        "microsoft",
        "azure",
        "google cloud storage",
        "gcs",
        "deep-learning",
        "machine-learning",
        "data-science",
        "neural-networks",
        "artificial-intelligence",
        "ai",
        "reinforcement-learning",
        "kubernetes",
        "aws",
        "microsoft",
        "azure",
        "google cloud",
        "tensorFlow",
        "pytorch",
    ],
    install_requires=[
        "click<8.0",
        "click-completion<0.6",
        "tabulate<=0.8.7",
        "Jinja2>=2.10.3",
        "kubernetes>=10.0.1",
        "marshmallow>=3.4.0,<=3.7.1",
        "polyaxon-sdk==1.1.6",
        "python-dateutil>=2.7.3",
        "pytz>=2019.2",
        "PyYAML>=5.1",
        "ujson>=1.35",
        "requests>=2.20.1",
        "requests-toolbelt>=0.8.0",
        "sentry-sdk>=0.12.3",
        "urllib3>=1.25.6",
        "certifi>=2019.9.11",
        "psutil",
        "nvidia-ml-py3",
    ],
    extras_require={
        "gcs": ["google-cloud-storage"],
        "s3": ["boto3", "botocore"],
        "azure": ["azure-storage-blob>=12.3.1"],
        "docker": ["docker"],
        "git": ["gitpython"],
        "polytune": ["scikit-learn==0.22.2", "hyperopt==0.2.4"],
        "polyboard": [
            "Pillow",
            "matplotlib<3.3.0",
            "moviepy",
            "plotly",
            "bokeh",
            "pandas",
            "altair",
        ],
        "streams": [
            "kubernetes_asyncio==11.2.0",
            "starlette==0.13.6",
            "aiofiles==0.5.0",
            "uvicorn==0.11.8",
            "pandas",
        ],
    },
    entry_points={"console_scripts": ["polyaxon = polyaxon.main:cli"]},
    python_requires=">=3.5",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    tests_require=["pytest"],
    cmdclass={"test": PyTest},
)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from tests.utils import BaseTestCase, assert_equal_dict

from polyaxon.polyflow.early_stopping import (
    V1FailureEarlyStopping,
    V1MetricEarlyStopping,
)
from polyaxon.polyflow.optimization import V1Optimization


@pytest.mark.workflow_mark
class TestEarlyStoppingConfigs(BaseTestCase):
    def test_metric_early_stopping(self):
        config_dict = {
            "kind": "metric_early_stopping",
            "metric": "loss",
            "value": 0.1,
            "optimization": V1Optimization.MAXIMIZE,
        }
        config = V1MetricEarlyStopping.from_dict(config_dict)
        config_to_dict = config.to_dict()
        assert config_to_dict.pop("optimization") == V1Optimization.MAXIMIZE
        assert_equal_dict(config_to_dict, config_dict)

    def test_metric_early_stopping_with_median_policy(self):
        config_dict = {
            "kind": "metric_early_stopping",
            "metric": "loss",
            "value": 0.1,
            "optimization": V1Optimization.MINIMIZE,
            "policy": {"kind": "median", "evaluationInterval": 1},
        }
        config = V1MetricEarlyStopping.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_metric_early_stopping_with_average_policy(self):
        config_dict = {
            "kind": "metric_early_stopping",
            "metric": "loss",
            "value": 0.1,
            "optimization": V1Optimization.MINIMIZE,
            "policy": {
                "kind": "diff",
                "evaluationInterval": 1,
                "minSamples": 3,
                "percent": 0.1,
            },
        }
        config = V1MetricEarlyStopping.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_metric_early_stopping_with_truncation_policy(self):
        config_dict = {
            "kind": "metric_early_stopping",
            "metric": "loss",
            "value": 0.1,
            "optimization": V1Optimization.MAXIMIZE,
            "policy": {"kind": "truncation", "percent": 0.5, "evaluationInterval": 1},
        }
        config = V1MetricEarlyStopping.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_failure_early_stopping_with_truncation_policy(self):
        config_dict = {
            "kind": "failure_early_stopping",
            "percent": 0.3,
        }
        config = V1FailureEarlyStopping.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from tests.utils import BaseTestCase

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import (
    V1BucketConnection,
    V1ClaimConnection,
    V1HostPathConnection,
    V1K8sResourceSchema,
)
from polyaxon.polypod.main.k8s_resources import get_requested_secrets
from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType


@pytest.mark.polypod_mark
class TestMainSecrets(BaseTestCase):
    def setUp(self):
        super().setUp()
        # Secrets
        self.resource1 = V1K8sResourceType(
            name="non_mount_test1",
            schema=V1K8sResourceSchema(
                name="non_mount_test1", items=["item1", "item2"]
            ),
            is_requested=False,
        )
        self.resource2 = V1K8sResourceType(
            name="non_mount_test2",
            schema=V1K8sResourceSchema(name="non_mount_test2"),
            is_requested=False,
        )
        self.resource3 = V1K8sResourceType(
            name="non_mount_test3",
            schema=V1K8sResourceSchema(
                name="non_mount_test3", items=["item1", "item2"]
            ),
            is_requested=True,
        )
        self.resource4 = V1K8sResourceType(
            name="non_mount_test4",
            schema=V1K8sResourceSchema(name="non_mount_test4"),
            is_requested=True,
        )
        self.resource5 = V1K8sResourceType(
            name="non_mount_test1",
            schema=V1K8sResourceSchema(name="non_mount_test1"),
            is_requested=True,
        )

        # Connections
        self.s3_store = V1ConnectionType(
            name="test_s3",
            kind=V1ConnectionKind.S3,
            schema=V1BucketConnection(bucket="s3//:foo"),
            secret=self.resource1.schema,
        )
        self.gcs_store = V1ConnectionType(
            name="test_gcs",
            kind=V1ConnectionKind.GCS,
            schema=V1BucketConnection(bucket="gcs//:foo"),
            secret=self.resource2.schema,
        )
        self.az_store = V1ConnectionType(
            name="test_az",
            kind=V1ConnectionKind.WASB,
            schema=V1BucketConnection(bucket="wasb://x@y.blob.core.windows.net"),
            secret=self.resource3.schema,
        )
        self.claim_store = V1ConnectionType(
            name="test_claim",
            kind=V1ConnectionKind.VOLUME_CLAIM,
            schema=V1ClaimConnection(mount_path="/tmp", volume_claim="test"),
        )
        self.host_path_store = V1ConnectionType(
            name="test_path",
            kind=V1ConnectionKind.HOST_PATH,
            schema=V1HostPathConnection(
                mount_path="/tmp", host_path="/tmp", read_only=True
            ),
        )

    def test_get_requested_secrets_non_values(self):
        assert get_requested_secrets(secrets=None, connections=None) == []
        assert get_requested_secrets(secrets=[], connections=[]) == []
        assert (
            get_requested_secrets(
                secrets=[self.resource1, self.resource2], connections=[]
            )
            == []
        )
        assert (
            get_requested_secrets(
                secrets=[], connections=[self.host_path_store, self.claim_store]
            )
            == []
        )

    def test_get_requested_secrets_and_secrets(self):
        expected = get_requested_secrets(secrets=[], connections=[self.s3_store])
        assert [e.schema for e in expected] == [self.resource1.schema]

        expected = get_requested_secrets(
            secrets=[self.resource2], connections=[self.s3_store]
        )
        assert [e.schema for e in expected] == [self.resource1.schema]

        expected = get_requested_secrets(
            secrets=[self.resource2], connections=[self.s3_store, self.gcs_store]
        )
        assert [e.schema for e in expected] == [
            self.resource1.schema,
            self.resource2.schema,
        ]

        expected = get_requested_secrets(
            secrets=[self.resource1, self.resource2],
            connections=[self.s3_store, self.gcs_store, self.az_store],
        )
        assert [e.schema for e in expected] == [
            self.resource1.schema,
            self.resource2.schema,
            self.resource3.schema,
        ]

    def test_get_requested_secrets(self):
        expected = get_requested_secrets(
            secrets=[self.resource1], connections=[self.s3_store]
        )
        assert [e.schema for e in expected] == [self.resource1.schema]
        expected = get_requested_secrets(
            secrets=[self.resource1, self.resource3], connections=[self.s3_store]
        )
        assert [e.schema for e in expected] == [
            self.resource3.schema,
            self.resource1.schema,
        ]
        expected = get_requested_secrets(
            secrets=[self.resource2, self.resource3, self.resource4],
            connections=[self.gcs_store],
        )
        assert [e.schema for e in expected] == [
            self.resource3.schema,
            self.resource4.schema,
            self.resource2.schema,
        ]
        expected = get_requested_secrets(
            secrets=[self.resource1, self.resource2], connections=[self.gcs_store]
        )
        assert [e.schema for e in expected] == [self.resource2.schema]
        expected = get_requested_secrets(
            secrets=[self.resource1, self.resource2],
            connections=[self.s3_store, self.gcs_store],
        )
        assert [e.schema for e in expected] == [
            self.resource1.schema,
            self.resource2.schema,
        ]
        expected = get_requested_secrets(
            secrets=[self.resource1, self.resource2],
            connections=[
                self.s3_store,
                self.gcs_store,
                self.host_path_store,
                self.claim_store,
            ],
        )
        assert [e.schema for e in expected] == [
            self.resource1.schema,
            self.resource2.schema,
        ]

        new_az_store = V1ConnectionType(
            name="test_az",
            kind=V1ConnectionKind.WASB,
            schema=V1BucketConnection(bucket="Conwasb://x@y.blob.core.windows.net"),
            secret=self.resource1,
        )
        expected = get_requested_secrets(
            secrets=[self.resource1, self.resource2],
            connections=[
                self.s3_store,
                self.gcs_store,
                new_az_store,
                self.host_path_store,
                self.claim_store,
            ],
        )
        assert [e.schema for e in expected] == [
            self.resource1.schema,
            self.resource2.schema,
        ]

        # Using a requested secret with same id
        expected = get_requested_secrets(
            secrets=[self.resource5, self.resource2],
            connections=[
                self.s3_store,
                self.gcs_store,
                new_az_store,
                self.host_path_store,
                self.claim_store,
            ],
        )
        assert [e.schema for e in expected] == [
            self.resource5.schema,
            self.resource2.schema,
        ]



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import math

try:
    import numpy as np
except ImportError:
    np = None


def sanitize_np_types(value):
    if isinstance(value, str):
        return value
    if math.isnan(value) or math.isinf(value):
        return None
    if isinstance(value, (int, float, complex, type(None))):
        return value
    if np and np.isnan(value):
        return None
    if np and isinstance(value, np.integer):
        return int(value)
    if np and isinstance(value, np.floating):
        return float(value)
    return value


def to_np(value):
    if isinstance(value, np.ndarray):
        return value
    if np.isscalar(value):
        return np.array([value])


def calculate_scale_factor(tensor):
    converted = tensor.numpy() if not isinstance(tensor, np.ndarray) else tensor
    return 1 if converted.dtype == np.uint8 else 255



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import asyncio

import click

from polyaxon.logger import logger
from polyaxon.utils.coroutine import coroutine


@click.command()
@click.option(
    "--container_id",
    "--container-id",
    help="The tagged image destination $PROJECT/$IMAGE:$TAG.",
)
@click.option(
    "--max_retries",
    "--max-retries",
    type=int,
    default=3,
    help="Number of times to retry the process.",
)
@click.option(
    "--sleep_interval",
    "--sleep-interval",
    type=int,
    default=2,
    help="Sleep interval between retries in seconds.",
)
@click.option(
    "--sync_interval",
    "--sync-interval",
    type=int,
    default=-1,
    help="Interval between artifacts syncs in seconds.",
)
@click.option(
    "--monitor-logs", is_flag=True, default=False, help="Enable logs monitoring.",
)
@coroutine
async def sidecar(
    container_id, max_retries, sleep_interval, sync_interval, monitor_logs
):
    """
    Start Polyaxon's sidecar command.
    """
    from polyaxon.sidecar import start_sidecar

    retry = 1
    while retry < max_retries:
        try:
            await start_sidecar(
                container_id=container_id,
                sleep_interval=sleep_interval,
                sync_interval=sync_interval,
                monitor_outputs=True,
                monitor_logs=monitor_logs,
            )
            return
        except Exception as e:
            logger.warning("Polyaxon sidecar retrying, error %s", e)
            retry += 1
            await asyncio.sleep(1 * retry)
